#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : nesc.
# @File         : hs
# @Time         : 2021/9/4 下午5:32
# @Author       : yuanjie
# @WeChat       : 313303303
# @Software     : PyCharm
# @Description  : 恒生系统-表结构解析

from meutils.pipe import *
from meutils.log_utils import logger4wecom
from meutils.decorators.catch import wecom_catch, wecom_hook


@wecom_catch()
def sheet2df(df, sheet=''):
    df = df.dropna(how='all').dropna(1, how='all')

    dfs = []
    col_map = {}

    for r in df.fillna(method='ffill').itertuples():
        if r._1 == '对象号':
            if col_map:
                dfs.append(
                    pd.DataFrame(col_map))  # {'对象号': '530500', '表名': 'v_his_businesstotal', '所在数据库': 'V8.11.2.0'}
                col_map = {}

            col_map.setdefault('对象号', r._2)

        if r._1 == '表名':
            col_map.setdefault('表名', r._2)

            if r._5 == '所在数据库':
                col_map.setdefault('所在数据库', r._6)

        if r._1 == '中文名':
            col_map.setdefault('中文名', r._2)

        if r._1 == '字段' and r._2 != '字段名':
            col_map.setdefault('字段名', []).append(r._2)
            col_map.setdefault('字段类型', []).append(r._3)
            col_map.setdefault('字段说明', []).append(r._5)

        if r._2.strip().startswith('idx_'):
            col_map['索引名称'] = col_map.get('索引名称', '') + ' ' + r._2
            col_map['索引字段'] = col_map.get('索引字段', '') + ' ' + r._5

    dfs.append(pd.DataFrame(col_map))
    df = pd.concat(dfs, ignore_index=True)
    df.insert(0, '表结构名', sheet)

    logger.info(f"{sheet}: {len(dfs)}个表")
    return df


if __name__ == '__main__':

    task = '恒生柜台系统表解析'

    logger.add(f'{task}.log')

    ps = Path('/Users/yuanjie/Downloads/UF20表结构/UF20表结构').glob('*.xls') | xlist

    dfs = []
    for p in tqdm(ps):
        df_map = pd.read_excel(
            p,
            header=0,
            sheet_name=None,
            # sheet_name=['表结构债券-债券账户']
        )

        logger.debug(f"文件名: {p.name} 有{len(df_map)}个sheet")

        sheet_list = []
        for sheet, df in tqdm(df_map.items()):
            sheet = sheet.strip()
            if sheet in ('数据库目录', '模块信息'):
                continue

            sheet_df = sheet2df(df, sheet)
            if sheet_df is None:
                logger.error(f"{p.name}__{sheet}解析错误！！！")
            else:
                sheet_list.append(sheet_df)

        df_all = pd.concat(sheet_list, ignore_index=True)

        df_all.insert(0, '文件名', p.name)

        dfs.append(df_all)

    df = pd.concat(dfs, ignore_index=True)

    df.to_csv(f"{task}.tsv", '\t', index=False)

    print(df.nunique())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : nesc.
# @File         : extract4ddl
# @Time         : 2021/9/6 下午12:28
# @Author       : yuanjie
# @WeChat       : 313303303
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *


def process_func(l):
    if '-' in l:
        字段说明 = re.split('-+', l)[1].strip()
        l = re.split('-+', l)[0].strip()
    else:
        字段说明 = None

    l = l.strip().strip(',').strip(' not null').strip().replace('\t', ' ')

    字段名 = l[0].split()[0]

    if '(' in l.split()[1]:
        数据类型 = l.split()[1].split('(')[0]
        数据长度 = l.split('(')[-1][:-1]
    else:
        数据类型 = l.split()[1]
        数据长度 = None

    return [字段名, 数据类型, 数据长度, 字段说明]


def extract_data(p, start_func=lambda x: x.startswith('create table'), end_func=lambda x: x.startswith('tablespace')):
    flag = False
    data_map = {}
    with open(p) as f:
        for r in tqdm(f):
            if flag:  # 左开右闭
                data_map.setdefault(table, []).append(r.strip())

            if start_func(r):
                table = r.strip().split()[-1]
                flag = True

            if end_func(r):
                flag = False
    return data_map


def main(ipath, opath=None):
    p = Path(ipath)
    if opath is None:
        opath = p.parent / f"{p.name}.tsv"
    d = extract_data(p)
    dd = {i: j[1:-2] for i, j in d.items()}

    df = pd.DataFrame(dd.items(), columns=['表名', '字段名'])
    df = df.explode('字段名', True)
    df[['字段名', '数据类型', '数据长度', '字段说明']] = df['字段名'].map(process_func).values.tolist()

    df.to_csv(opath, '\t', index=False)

    logger.info(bjson(dict(zip(df.nunique().index, df.nunique()))))

    return


if __name__ == '__main__':
    main('/Users/yuanjie/Desktop/notebook/0_TODO/mot_part.sql')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : __init__.py
# @Time         : 2021/1/31 10:20 下午
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : python meutils/clis/__init__.py


import typer

from meutils.pipe import *

cli = typer.Typer(name="东北证券 clis")


def _run_cmd(cmd, nohup=0):
    cmd = f"nohup {cmd} &" if nohup else cmd
    logger.debug(cmd)
    return os.system(cmd)


@cli.command(help="help")  # help会覆盖docstring
def clitest(name: str):
    """

    @param name: name
    @return:
    """
    typer.echo(f"Hello {name}")


@cli.command()
def run(ipath: str, opath=None):
    from nesc.extract import extract4ddl
    extract4ddl.main(ipath, opath)


if __name__ == '__main__':
    cli()
